import random
import time
import pyglet
import endscreen
from pyglet.window import key


class GameEventHandlers:
    def __init__(self, window):
        self.window = window

    def on_draw(self):
        self.window.clear()

    def on_key_press(self, symbol, modifiers):
        if symbol == key.U:
            self.on_draw()


class Sprite(pyglet.sprite.Sprite):
    def __init__(self, race, sex, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.race = race
        self.sex = sex
        self.is_stopped = False
        self.has_drugs = random.randint(0, 1)

    def centre(self):
        return self.x + (self.width/2), self.y + (self.height/2)


def _get_walk_sprite(sprite_sheet, batch=None, group=None):
    _image_grid = pyglet.image.ImageGrid(sprite_sheet['image'], rows=5, columns=9)
    _ani = pyglet.image.Animation.from_image_sequence(_image_grid,
                                                      duration=0.1)
    _ani.frames = _ani.frames[:8]
    return Sprite(sprite_sheet['race'], sprite_sheet['sex'], _ani,
                  batch=batch, group=group)


def game(window):
    global group_order
    group_order = 1
    global searching
    searching = False
    spawned = []
    searched = []
    batch = pyglet.graphics.Batch()
    _street_image = pyglet.resource.image('street.png')
    street = pyglet.sprite.Sprite(_street_image, x=0, y=0)
    street.scale = 0.5
    characters = []
    ui_group = pyglet.graphics.OrderedGroup(1000)
    jailed = []
    in_jail = pyglet.text.Label('In jail:',
                                font_name='Syncopate',
                                font_size=12,
                                x=0, y=window.height-20,
                                batch=batch, group=ui_group)
    num_in_jail_label = pyglet.text.Label(str(len(jailed)),
                                          font_name='Syncopate',
                                        font_size=12,
                                          x=80, y=window.height-20,
                                          batch=batch, group=ui_group)
    character_types = [
        {'image': pyglet.resource.image('character_femalePerson_sheet.png'),
         'race': 'black',
         'sex': 'female'},
        {'image': pyglet.resource.image('character_malePerson_sheet.png'),
         'race': 'black',
         'sex': 'male'},
        {'image': pyglet.resource.image('character_femaleAdventurer_sheet.png'),
         'race': 'white',
         'sex': 'female'},
        {'image': pyglet.resource.image('character_maleAdventurer_sheet.png'),
         'race': 'white',
         'sex': 'male'}]
    game_end = time.time() + 60

    def update(dt):
        if time.time() > game_end:
            pyglet.clock.unschedule(update)
            pyglet.clock.unschedule(spawn_characters)
            endscreen.end_screen(window, spawned, jailed, searched)
        for character in characters:
            if character.is_stopped:
                if character.time_stopped + 4 > time.time():
                    time_left = character.time_stopped + 4 - time.time()
                    character.timer.text = str(int(time_left))
                else:
                    character.color = (255, 255, 255)
                    character.timer.delete()
                    character.searched_bar.delete()
                    character.is_stopped = False
                    global searching
                    searching = False
                character.searched_bar.height = 20 - character.percent_searched/5
            else:
                character.y -= 60*dt
                _pre_width = character.width
                character.scale += dt/10
                character.x -= (character.width - _pre_width)/2
                cx, cy = character.centre()
                if cx > (window.width/2):
                    character.x += dt*2.5
                else:
                    character.x -= dt*2.5
                if character.y <= -character.height:
                    character.delete()
                    characters.remove(character)
    pyglet.clock.schedule(update)

    def spawn_characters(dt, force_spawn=False):
        global group_order
        if force_spawn or random.randint(0, 1):
            _group = pyglet.graphics.OrderedGroup(group_order)
            group_order -= 1
            char_type = random.sample(character_types, 1)[0]
            characters.append(
                _get_walk_sprite(
                    char_type, batch=batch,
                    group=_group))
            characters[-1].update(
                x=random.randrange(window.width//3, 2*window.width//3),
                y=window.height+characters[-1].height/2, scale=0.5)
            print(len(characters))
            spawned.append({'race': characters[-1].race,
                            'sex': characters[-1].sex,
                            'has_drugs': characters[-1].has_drugs})
    pyglet.clock.schedule_interval(spawn_characters, 1)

    def check_cursor_on_character(character, x, y):
        left = character.x
        right = character.x + character.width
        top = character.y + character.height
        bottom = character.y
        if (left < x < right and bottom < y < top):
            return character
        else:
            return None

    def do_stop(x, y):
        global searching
        stopped_char = None
        if searching and check_cursor_on_character(searching, x, y):
            return searching
        for character in characters:
            left = character.x
            right = character.x + character.width
            top = character.y + character.height
            bottom = character.y
            if (left < x < right and bottom < y < top):
                stopped_char = character
        return stopped_char

    def search(character):
        character.color = (254, 0, 0)
        character.percent_searched = 0
        print('Stopped Character!')

    def keep_searching(character):
        global searching
        character.time_stopped = time.time()
        character.percent_searched += 5
        if character.percent_searched >= 100:
            searched.append({'race': character.race,
                             'sex': character.sex})
            if character.has_drugs:
                jailed.append({'race': character.race,
                               'sex': character.sex})
                num_in_jail_label.text = str(len(jailed))
                stop_searching(character, delete=True)
            else:
                stop_searching(character, color=(0, 127, 0))

    def stop_searching(character, color=(255, 255, 255), delete=False):
        global searching
        searching = False
        character.is_stopped = False
        character.timer.delete()
        character.searched_bar.delete()
        character.color = color
        if delete:
            characters.remove(character)
            character.delete()

    @window.event
    def on_mouse_press(x, y, button, modifiers):
        global searching
        if button == pyglet.window.mouse.LEFT:
            stopped_character = do_stop(x, y)
            if searching and stopped_character == searching:
                keep_searching(searching)
            else:
                if stopped_character:
                    if searching:
                        stop_searching(searching)
                    stopped_character.is_stopped = True
                    searching = stopped_character
                    stopped_character.time_stopped = time.time()
                    stopped_character.timer = pyglet.text.Label(
                        '4',
                        font_name='Syncopate',
                        font_size=12,
                        x=stopped_character.x,
                        y=stopped_character.y,
                        batch=batch, group=ui_group)
                    search(stopped_character)
                    stopped_character.searched_bar = pyglet.shapes.Rectangle(
                        x=stopped_character.x+20,
                        y=stopped_character.y,
                        width=20,
                        height=20,
                        batch=batch, group=ui_group)

    @window.event
    def on_draw():
        window.clear()
        street.draw()
        batch.draw()
        # street.draw()
        # fem_person.draw()

    spawn_characters(0, force_spawn=True)
